module Util

import Data.Vect

%default total
%access public export

---------------------
namespace NonZeroNat
    data NonZeroNat : Type where
        MkNonZeroNat : (value : Nat) -> {auto prf : Prelude.Nat.GT value Z} -> NonZeroNat

extractNonZeroNat : NonZeroNat -> Nat
extractNonZeroNat (MkNonZeroNat value) = value


---------------------
namespace ForEveryElementVect
    data ForEveryElementVect : {A : Type} -> (vect : Vect n A) -> (predicate : A -> Type) -> Type where
        EmptyListOk : ForEveryElementVect [] predicate
        NextElOk : {A : Type} -> (prev : ForEveryElementVect {A} xs predicate) -> (prf : predicate el) -> ForEveryElementVect {A} (el :: xs) predicate


    proofContraHead :
        {A : Type}
        -> (contraHead : predicate x -> Void)
        -> (decisionProc : (a : A) -> Dec (predicate a))
        -> ForEveryElementVect (x :: xs) predicate -> Void
    proofContraHead contraHead decisionProc prfEvery = case prfEvery of
        (NextElOk prev prfX) => contraHead prfX

    proofContraTail :
        {A : Type}
        -> (decisionProc : (a : A) -> Dec (predicate a))
        -> (prfHead : predicate x)
        -> (contraTail : ForEveryElementVect xs predicate -> Void)
        -> ForEveryElementVect (x :: xs) predicate -> Void
    proofContraTail decisionProc prfHead contraTail prfEvery = case prfEvery of
        (NextElOk prev prf) => contraTail prev

    forEveryElementVect :
        {A : Type}
        -> (vect : Vect n A)
        -> (predicate : A -> Type)
        -> (decisionProc : (a : A) -> Dec (predicate a))
        -> Dec (ForEveryElementVect {A} vect predicate)

    forEveryElementVect [] predicate decisionProc = Yes EmptyListOk
    forEveryElementVect (x :: xs) predicate decisionProc = case (decisionProc x) of
        (No contraHead) => No (proofContraHead contraHead decisionProc)
        (Yes prfHead) =>
            case (forEveryElementVect xs predicate decisionProc) of
                (No contraTail) => No (proofContraTail decisionProc prfHead contraTail)
                (Yes prfTail) => Yes $ NextElOk prfTail prfHead
