//---------------------------------------------------------------------------------------------------------------------
const MixinIdentity = Symbol('MixinIdentity');
const isInstanceOfStatic = function (instance) {
    return Boolean(instance && instance[this[MixinIdentity]]);
};
// we want the return type to match the type of the argument
// this does not work:
//      export const instanceOf = <T extends AnyConstructor<AnyFunction<object>>(base : T) : T => {
// the mixins wrapped with such declaration loses the type of return value
// because of this we just apply typecasting to the argument
export const instanceOf = (arg) => {
    const mixin = arg;
    const symbol = Symbol(mixin.name);
    const wrapper = function (base) {
        const extendedClass = mixin(base);
        extendedClass.prototype[symbol] = true;
        return extendedClass;
    };
    wrapper[MixinIdentity] = symbol;
    Object.defineProperty(wrapper, Symbol.hasInstance, { value: isInstanceOfStatic });
    return wrapper;
};
//---------------------------------------------------------------------------------------------------------------------
// the `instanceof` analog with typeguard:
// https://www.typescriptlang.org/docs/handbook/advanced-types.html#user-defined-type-guards
export const isInstanceOf = (instance, func) => {
    return Boolean(instance && instance[func[MixinIdentity]]);
};
//---------------------------------------------------------------------------------------
const ClassIdentity = Symbol('ClassIdentity');
let counter = 0;
const getClassId = (cls) => {
    let classId = cls[ClassIdentity];
    if (!classId)
        classId = cls[ClassIdentity] = ++counter;
    return classId;
};
const classCache = new Map();
export const buildClass = (base, ...mixins) => {
    const classId = mixins.reduce((classId, mixin) => classId + '/' + getClassId(mixin), String(getClassId(base)));
    let cls = classCache.get(classId);
    if (!cls) {
        cls = mixins.reduce((klass, mixin) => mixin ? mixin(klass) : klass, base);
        classCache.set(classId, cls);
    }
    return cls;
};
